import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/foundation.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_user/ocg_user.dart';
import 'package:ecommerce_profile/src/models/customer.dart';
import 'package:ecommerce_profile/src/repository/customer/base.dart';

class CustomerFirebaseRepository extends CustomerRepository {
  final IDHelper idHelper;
  final FirebaseFirestore _firestore;
  final UserRepository userRepository;
  String get customers => "customers";
  CollectionReference get customerCollection =>
      _firestore.collection(customers);
  CustomerFirebaseRepository(this.idHelper, this.userRepository)
      : _firestore = FirebaseFirestore.instance;
  Future<AppResponse<String>> create(Customer customer) async {
    try {
      if (InputHelper.isBlank(customer.name)) {
        return AppResponse.named(error: "Name can not be empty");
      }
      var appResponse = await userRepository.getSelfId();
      if (!appResponse.isSuccess) {
        return AppResponse.named(error: "Not logged in");
      }
      customer = customer.copyWith(userId: appResponse.data);
      var oldDocuments = await customerCollection
          .where("userId", isEqualTo: customer.userId)
          .limit(1)
          .get();
      if (oldDocuments.docs.length > 0) {
        return AppResponse.named(
            error: "Can not have more customer with same userId");
      }
      String id = idHelper.generateId();
      customer = customer.copyWith(id: id);
      var json = customer.toJson();
      await customerCollection.doc(id).set(json);
      return AppResponse.named(data: id);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  Future<AppResponse<bool>> delete(String id) async {
    try {
      await customerCollection.doc(id).delete();
      return AppResponse.named(data: true);
    } catch (e) {
      debugPrint("Error $e");
      return AppResponse.named(error: "$e");
    }
  }

  Future<AppResponse<List<Customer>>> getList(
      {int page = 1, int count = 10}) async {
    try {
      var snapshot = await customerCollection.get();
      if (snapshot.docs.length > 0) {
        var list = await Future.wait(snapshot.docs.map((e) async {
          return await _parseJson(e.data());
        }));
        list = list.where((element) => element != null).toList();
        return AppResponse.named(data: list);
      }
      return AppResponse.named(data: []);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  Future<AppResponse<Customer>> read(String id) async {
    try {
      var snapshot = await customerCollection.doc(id).get();
      if (snapshot.exists) {
        return AppResponse.named(data: await _parseJson(snapshot.data()));
      } else {
        return AppResponse.named(error: "Not exists");
      }
    } catch (e) {
      debugPrint("Error $e");
      return AppResponse.named(error: "$e");
    }
  }

  Future<Customer> _parseJson(var json) async {
    return Customer.fromJson(json);
  }

  Future<AppResponse<bool>> update(Customer object) async {
    try {
      await _firestore.doc(object.id).update(object.toJson());
      return AppResponse.named(data: true);
    } catch (e) {
      return AppResponse.named(error: "Error $e");
    }
  }

  @override
  Future<AppResponse<Customer>> getByUser(String userId) async {
    try {
      var snapshot = await customerCollection
          .where("userId", isEqualTo: userId)
          .limit(1)
          .get();
      if (snapshot.docs.length > 0) {
        return AppResponse.named(
            data: await _parseJson(snapshot.docs[0].data()));
      } else {
        return AppResponse.named(error: "Not exists");
      }
    } catch (e) {
      debugPrint("Error $e");
      return AppResponse.named(error: "$e");
    }
  }
}
