import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/foundation.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_user/ocg_user.dart';
import 'package:ecommerce_profile/src/models/store.dart';
import 'package:ecommerce_profile/src/repository/store/base.dart';

class StoreFirebaseRepository extends StoreRepository {
  final IDHelper idHelper;
  final FirebaseFirestore _firestore;
  final UserRepository userRepository;
  String get stores => "stores";
  CollectionReference get storeCollection => _firestore.collection(stores);
  StoreFirebaseRepository(this.idHelper, this.userRepository)
      : _firestore = FirebaseFirestore.instance;
  Future<AppResponse<String>> create(Store store) async {
    try {
      if (InputHelper.isBlank(store.name)) {
        return AppResponse.named(error: "Name can not be empty");
      }
      var appResponse = await userRepository.getSelfId();
      debugPrint("Response ${appResponse.data}");
      if (!appResponse.isSuccess) {
        return AppResponse.named(error: "Not logged in");
      }
      store = store.copyWith(userId: appResponse.data);
      var oldDocuments = await storeCollection
          .where("userId", isEqualTo: store.userId)
          .limit(1)
          .get();
      if (oldDocuments.docs.length > 0) {
        return AppResponse.named(
            error: "Can not have more store with same userId");
      }
      String id = idHelper.generateId();
      store = store.copyWith(id: id);
      var json = store.toJson();
      await storeCollection.doc(id).set(json);
      return AppResponse.named(data: id);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  Future<AppResponse<bool>> delete(String id) async {
    try {
      await storeCollection.doc(id).delete();
      return AppResponse.named(data: true);
    } catch (e) {
      debugPrint("Error $e");
      return AppResponse.named(error: "$e");
    }
  }

  Future<AppResponse<List<Store>>> getList(
      {int page = 1, int count = 10}) async {
    try {
      var snapshot = await storeCollection.get();
      if (snapshot.docs.length > 0) {
        var list = await Future.wait(snapshot.docs.map((e) async {
          return await _parseJson(e.data());
        }));
        list = list.where((element) => element != null).toList();
        return AppResponse.named(data: list);
      }
      return AppResponse.named(data: []);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  Future<AppResponse<Store>> read(String id) async {
    try {
      var snapshot = await storeCollection.doc(id).get();
      if (snapshot.exists) {
        return AppResponse.named(data: await _parseJson(snapshot.data()));
      } else {
        return AppResponse.named(error: "Not exists");
      }
    } catch (e) {
      debugPrint("Error $e");
      return AppResponse.named(error: "$e");
    }
  }

  Future<Store> _parseJson(var json) async {
    // TODO
    json["categories"] = [];
    return Store.fromJson(json);
  }

  Future<AppResponse<bool>> update(Store object) async {
    try {
      await _firestore.doc(object.id).update(object.toJson());
      return AppResponse.named(data: true);
    } catch (e) {
      return AppResponse.named(error: "Error $e");
    }
  }

  @override
  Future<AppResponse<Store>> getByUser(String userId) async {
    try {
      var snapshot = await storeCollection
          .where("userId", isEqualTo: userId)
          .limit(1)
          .get();
      if (snapshot.docs.length > 0) {
        return AppResponse.named(
            data: await _parseJson(snapshot.docs[0].data()));
      } else {
        return AppResponse.named(error: "Not exists");
      }
    } catch (e) {
      debugPrint("Error $e");
      return AppResponse.named(error: "$e");
    }
  }
}
